@@ -41,6 +41,7 @@ namespace Beagle.Filters {
 		public FilterSpreadsheet () 
 		{
 			SnippetMode = true;
+			SetFileType ("document");
 		}
 
 		protected override void RegisterSupportedTypes ()
@@ -82,6 +83,7 @@ namespace Beagle.Filters {
 			SafeProcess pc = new SafeProcess ();
 			pc.Arguments = new string [] { "ssindex", "-i", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
+			pc.UseLangC = true;
 
 			// Let ssindex run for 10 seconds, max.
 			pc.CpuLimit = 10;
