@@ -703,7 +703,7 @@ namespace Beagle.Util {
 			bool need_throttle = false;
 
 			// Never speed up if we are using the battery.
-			if (idle_time > idle_threshold && ! SystemInformation.UsingBattery) {
+			if (idle_time > idle_threshold && ! Battery.UsingBattery) {
 				is_idle = true;
 				double t = (idle_time - idle_threshold) / idle_ramp_up_time;				     
 				idle_scale = (1 - Math.Min (t, 1.0));
