@@ -26,10 +26,12 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.Globalization;
 using System.IO;
 using System.Text;
 using System.Xml;
+using System.Text.RegularExpressions;
 
 using Mono.Unix;
 
@@ -46,8 +48,16 @@ namespace Beagle.Util {
 		public static DateTime MinValueUtc = new DateTime (0, DateTimeKind.Utc);
 		public static DateTime MaxValueUtc = new DateTime (DateTime.MaxValue.Ticks, DateTimeKind.Utc);
 
-		// We use this instead of DateTime.ToUniversalTime() because
-		// we want to assume DateTimeKind.Unspecified dates are UTC
+		/// <summary>
+		/// We use this instead of DateTime.ToUniversalTime() because
+		/// we want to assume DateTimeKind.Unspecified dates are UTC
+		/// </summary>
+		/// <param name="dt">
+		/// A <see cref="DateTime"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="DateTime"/>
+		/// </returns>
 		static private DateTime ToUniversalTime (DateTime dt)
 		{
 			switch (dt.Kind) {
@@ -87,20 +97,21 @@ namespace Beagle.Util {
 
                 static public DateTime StringToDateTime (string str)
                 {
-			if (str == null || str == String.Empty)
+			if (String.IsNullOrEmpty (str) || str.Length != 14)
 				return new DateTime ();
 
-			// FIXME: Workaround for http://bugzilla.ximian.com/show_bug.cgi?id=80320
-			// ArgumentOutOfRangeException is incorrectly thrown for
-			// DateTime.MinValue.ToLocalTime() in timezones with a negative offset
-			// from UTC, and DateTime.MinValue.ToUniversalTime() in timezones with a
-			// positive offset from UTC.  Note that Mono works correctly for going
-			// beyond MaxValue, so we only need to deal with the lower bound.
-			try {
-				return DateTime.ParseExact (str, TimeFormat, CultureInfo.InvariantCulture, DateTimeStyles.AssumeUniversal | DateTimeStyles.AdjustToUniversal);
-			} catch (ArgumentOutOfRangeException) {
-				return DateTime.MinValue;
-			}
+			int year, month, day, hour, min, sec;
+
+			// Pretty simple format, parse it ourself
+			year  = int.Parse (str.Substring (0, 4));
+			month = int.Parse (str.Substring (4, 2));
+			day   = int.Parse (str.Substring (6, 2));
+			hour  = int.Parse (str.Substring (8, 2));
+			min   = int.Parse (str.Substring (10, 2));
+			sec   = int.Parse (str.Substring (12, 2));
+
+			DateTime dt = new DateTime (year, month, day, hour, min, sec, DateTimeKind.Utc);
+			return dt;
                 }
 
 		static public string DateTimeToFuzzy (DateTime dt)
@@ -225,12 +236,20 @@ namespace Beagle.Util {
 			return String.Format (Catalog.GetString ("{0:0.0} MB"), len/(double)oneMb);
 		}
 
-		// Here we:
-		// (1) Replace non-alphanumeric characters with spaces
-		// (2) Inject whitespace between lowercase-to-uppercase
-		//     transitions (so "FooBar" becomes "Foo Bar")
-		//     and transitions between letters and numbers
-		//     (so "cvs2svn" becomes "cvs 2 svn")
+		/// <summary>
+		///  Here we:
+		/// (1) Replace non-alphanumeric characters with spaces
+		/// (2) Inject whitespace between lowercase-to-uppercase
+		///     transitions (so "FooBar" becomes "Foo Bar")
+		///     and transitions between letters and numbers
+		///     (so "cvs2svn" becomes "cvs 2 svn")
+		/// </summary>
+		/// <param name="line">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.String"/>
+		/// </returns>
 		static public string FuzzyDivide (string line)
 		{
 			// Allocate a space slightly bigger than the
@@ -280,35 +299,117 @@ namespace Beagle.Util {
 			return FuzzyDivide (url.Substring (protocol_index + 3));
 		}
 		
-		// Match strings against patterns that are allowed to contain
-		// glob-style * wildcards.
-		// This recursive implementation is not particularly efficient,
-		// and probably will fail for weird corner cases.
+		/// <summary>
+		/// 	 Match strings against patterns that are allowed to contain
+		///	 glob-style * and ? wildcards.
+		/// </summary>
+		/// <param name="pattern">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <param name="str">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
 		static public bool GlobMatch (string pattern, string str)
 		{
 			if (pattern == null || str == null)
 				return false;
-
-			if (pattern == "*")
+			else if (pattern == "*")
 				return true;
-			else if (pattern.StartsWith ("**"))
-				return GlobMatch (pattern.Substring (1), str);
-			else if (str == "" && pattern != "")
+			else if (str == String.Empty && pattern != String.Empty)
 				return false;
-
-			int i = pattern.IndexOf ('*');
-			if (i == -1)
+			else if (pattern.IndexOf ('*') == -1)
 				return pattern == str;
-			else if (i > 0 && i < str.Length)
-				return pattern.Substring (0, i) == str.Substring (0, i)
-					&& GlobMatch (pattern.Substring (i), str.Substring (i));
-			else if (i == 0)
-				return GlobMatch (pattern.Substring (1), str.Substring (1))
-					|| GlobMatch (pattern.Substring (1), str)
-					|| GlobMatch (pattern, str.Substring (1));
+			else
+				return WildcardEquals (pattern, 0, str, 0);
+		}
+		
+		private const char WILDCARD_STRING = '*';
+		
+		/// <summary>
+		/// Copied from beagled/Lucene.Net/Search/WildcardTermEnum.cs
+		/// Simple string matching algorithm with wildcards
+		/// '*' matches 0 or more characters
+		/// </summary>
+		/// <param name="pattern">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <param name="patternIdx">
+		/// A <see cref="System.Int32"/>
+		/// </param>
+		/// <param name="text">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <param name="stringIdx">
+		/// A <see cref="System.Int32"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Boolean"/>
+		/// </returns>
+		private static bool WildcardEquals(System.String pattern, int patternIdx, System.String text, int stringIdx)
+		{
+			int p = patternIdx;
 			
+			for (int s = stringIdx; ; ++p, ++s) {
+				// End of string yet?
+				bool sEnd = (s >= text.Length);
+				// End of pattern yet?
+				bool pEnd = (p >= pattern.Length);
+
+				// If we're looking at the end of the string...
+				if (sEnd) {
+					// Assume the only thing left on the pattern is/are wildcards
+					bool justWildcardsLeft = true;
+					
+					// Current wildcard position
+					int wildcardSearchPos = p;
+					// While we haven't found the end of the pattern,
+					// and haven't encountered any non-wildcard characters
+					while (wildcardSearchPos < pattern.Length && justWildcardsLeft) {
+						// Check the character at the current position
+						char wildchar = pattern[wildcardSearchPos];
+						
+						// If it's not a wildcard character, then there is more
+						// pattern information after this/these wildcards.
+						if (wildchar != WILDCARD_STRING)
+							justWildcardsLeft = false;
+						else
+							// Look at the next character
+							wildcardSearchPos++;
+					}
+					
+					// This was a prefix wildcard search, and we've matched, so
+					// return true.
+					if (justWildcardsLeft) {
+						return true;
+					}
+				}
+				
+				// If we've gone past the end of the string, or the pattern,
+				// return false.
+				if (sEnd || pEnd) {
+					break;
+				}
+				
+				if (pattern[p] == WILDCARD_STRING) {
+					// Look at the character beyond the '*'.
+					++p;
+					// Examine the string, starting at the last character.
+					for (int i = text.Length; i >= s; --i) {
+						if (WildcardEquals(pattern, p, text, i)) {
+							return true;
+						}
+					}
+					break;
+				}
+				if (pattern[p] != text[s]) {
+					break;
+				}
+			}
 			return false;
-		} 
+		}
 
 		// FIXME: how do we do this operation in a culture-neutral way?
 		static public string[] SplitQuoted (string str)
@@ -359,9 +460,14 @@ namespace Beagle.Util {
 
 		static public string HexEscape (string str)
 		{
-			StringBuilder builder = new StringBuilder ();
+			int index = -1;
+			if ((index = str.IndexOfAny (CharsToQuote)) == -1)
+				return str;
 
-			foreach (char c in str) {
+			StringBuilder builder = new StringBuilder (str, 0, index, str.Length << 1);
+
+			for (; index < str.Length; ++ index) {
+				char c = str [index];
 
 				if (ArrayFu.IndexOfChar (CharsToQuote, c) != -1)
 					builder.Append (Uri.HexEscape (c));
@@ -380,26 +486,37 @@ namespace Beagle.Util {
 			return builder.ToString ();
 		}
 
-		// Translate all %xx codes into real characters
+		/// <summary>
+		/// Translate all %xx codes into real characters
+		/// </summary>
+		/// <param name="str">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.String"/>
+		/// </returns>
 		static public string HexUnescape (string str)
 		{
-			ArrayList bytes = new ArrayList ();
-                        byte[] sub_bytes;
                         int i, pos = 0;
+			if ((i = str.IndexOf ('%')) == -1)
+				return str;
 
-                        while ((i = str.IndexOf ('%', pos)) != -1) {
+			List<byte> bytes = new List<byte> (str.Length);
+                        byte[] sub_bytes;
+
+			do {
                                 sub_bytes = Encoding.UTF8.GetBytes (str.Substring (pos, i - pos));
                                 bytes.AddRange (sub_bytes);
 				
 				pos = i;
                                 char unescaped = Uri.HexUnescape (str, ref pos);
-				bytes.Add ((byte) unescaped);
-                        }
+				bytes.Add (Convert.ToByte (unescaped));
+                        } while ((i = str.IndexOf ('%', pos)) != -1);
 
                         sub_bytes = Encoding.UTF8.GetBytes (str.Substring (pos, str.Length - pos));
                         bytes.AddRange (sub_bytes);
 
-                        return Encoding.UTF8.GetString ((byte[]) bytes.ToArray (typeof (byte)));
+                        return Encoding.UTF8.GetString (bytes.ToArray ());
 		}
 
 		// These strings should never be exposed to the user.
@@ -549,7 +666,21 @@ namespace Beagle.Util {
 			return new string (char_array);
 		}
 		
-		// Words of less than min_word_length characters are not counted
+		/// <summary>
+		/// Words of less than min_word_length characters are not counted
+		/// </summary>
+		/// <param name="str">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <param name="max_words">
+		/// A <see cref="System.Int32"/>
+		/// </param>
+		/// <param name="min_word_length">
+		/// A <see cref="System.Int32"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.Int32"/>
+		/// </returns>
 		static public int CountWords (string str, int max_words, int min_word_length)
 		{
 			if (str == null)
@@ -589,7 +720,15 @@ namespace Beagle.Util {
 			return CountWords (str, -1);
 		}
 
-		// Strip trailing slashes and make sure we only have 1 leading slash
+		/// <summary>
+		/// Strip trailing slashes and make sure we only have 1 leading slash
+		/// </summary>
+		/// <param name="path">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.String"/>
+		/// </returns>
 		static public string SanitizePath (string path)
 		{
 			if (path.StartsWith ("//")) {
@@ -606,12 +745,20 @@ namespace Beagle.Util {
 			return path;
 		}
 
-		// This method will translate an email address like
-		// "john.doe+spamtrap@foo.com" to "john doe spamtrap foo"
-		//
-		// FIXME: Maybe we should only do the username part?  Ie,
-		// "john doe spamtrap"?  That way searching for "foo" won't
-		// turn up *everything*
+		/// <summary>
+		/// This method will translate an email address like
+		/// "john.doe+spamtrap@foo.com" to "john doe spamtrap foo"
+		///
+		/// FIXME: Maybe we should only do the username part?  Ie,
+		/// "john doe spamtrap"?  That way searching for "foo" won't
+		/// turn up *everything*
+		/// </summary>
+		/// <param name="email">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.String"/>
+		/// </returns>
 		static public string SanitizeEmail (string email)
 		{
 			char[] replace_array = { '@', '.', '-', '_', '+' };
@@ -635,10 +782,17 @@ namespace Beagle.Util {
 			return email;
 		}
 
-		/**
-		 * expands environment variables in a string e.g.
-		 * folders=$HOME/.kde/share/...
-		 */
+		
+		/// <summary>
+		/// expands environment variables in a string e.g.
+		/// folders=$HOME/.kde/share/...
+		/// </summary>
+		/// <param name="path">
+		/// A <see cref="System.String"/>
+		/// </param>
+		/// <returns>
+		/// A <see cref="System.String"/>
+		/// </returns>
 		public static string ExpandEnvVariables (string path)
 		{
 			int dollar_pos = path.IndexOf ('$');
@@ -647,7 +801,9 @@ namespace Beagle.Util {
 			
 			System.Text.StringBuilder sb = 
 				new System.Text.StringBuilder ( (dollar_pos == 0 ? "" : path.Substring (0, dollar_pos)));
-			
+
+			bool env_var_found = false;
+
 			while (dollar_pos != -1 && dollar_pos + 1 < path.Length) {
 				// FIXME: kconfigbase.cpp contains an additional case, $(expression)/.kde/...
 				// Ignoring such complicated expressions for now. Volunteers ;) ?
@@ -669,11 +825,15 @@ namespace Beagle.Util {
 							end_pos ++;
 						var_name = path.Substring (dollar_pos + 1, end_pos - dollar_pos - 1);
 					}
+
 					string value_env = null;
 					if (var_name != String.Empty)
 						value_env = Environment.GetEnvironmentVariable (var_name);
 					if (value_env != null) {
 						sb.Append (value_env);
+						env_var_found = true;
+					} else {
+						env_var_found = false;
 					}
 					// else, no environment variable with that name exists. ignore
 				}else // else, ignore the first '$', second one will be expanded
@@ -688,6 +848,9 @@ namespace Beagle.Util {
 				}
 			}
 
+			if (! env_var_found)
+				return null;
+
 			return sb.ToString ();
 		}
 
@@ -741,44 +904,58 @@ namespace Beagle.Util {
 		
 		public static string ConvertSpecialEntities (string line)
 		{
-			line.Replace ("&lt;", "<");
-			line.Replace ("&gt;", ">");
-			line.Replace ("&quot;", "\"");
-			line.Replace ("&amp;", "&");
-			line.Replace ("&nbsp", " ");
+			line = line.Replace ("&lt;", "<");
+			line = line.Replace ("&gt;", ">");
+			line = line.Replace ("&quot;", "\"");
+			line = line.Replace ("&amp;", "&");
+			line = line.Replace ("&nbsp;", " ");
+			line = line.Replace ("&apos;", "\'");
 
 			return line;
   		}
-	}
 
-	public class HtmlRemovingReader : TextReader {
-
-		private TextReader reader;
-		private StringBuilder sb;
-	    
-		public HtmlRemovingReader (TextReader reader)
+		public static Regex GetPatternRegex (IEnumerable patterns)
 		{
-			this.reader = reader;
-			this.sb = new StringBuilder ();
-		}
-
-		public override string ReadLine ()
-		{
-			string line = reader.ReadLine ();
-
-			if (line == null)
+			if (patterns == null)
 				return null;
 
-			sb.Length = 0;
-			line = StringFu.StripTags (line, sb);
-			line = StringFu.ConvertSpecialEntities (line);
+			StringBuilder sb = new StringBuilder ();
+			bool first = true;
+			Regex tmp_regex = null;
+			foreach (string pattern in patterns) {
+				try {
+					string regex_pattern;
+					if (pattern.Length >= 2 && pattern [0] == '/' && pattern.EndsWith ("/")) {
+						regex_pattern = pattern.Substring (1, pattern. Length - 2);
+					} else {
+						// Make *.foo to .*\.foo
+						regex_pattern = pattern.Replace (".", @"\.");
+						regex_pattern = regex_pattern.Replace ("*", ".*");
+					}
+					tmp_regex = new Regex (regex_pattern);
+					sb.Append (String.Format (@"{0}^(?:{1})$", first ? String.Empty : "|", regex_pattern));
+				} catch (System.ArgumentException) {
+					Log.Warn ("Ignoring invalid regex: [{0}]", pattern);
+				}
+				first = false;
+			}
 
-			return line;
+			// If no patterns are there, prevent generating a regex that will match everything
+			if (sb.Length == 0)
+				return null;
+
+			//Log.Debug ("regex: [{0}]", sb.ToString());
+			return new Regex (sb.ToString (), RegexOptions.Compiled);
 		}
 
-		public override void Close ()
-		{
-			reader.Close ();
+		public class OrdinalComparer : IComparer {
+
+			public static OrdinalComparer Instance = new OrdinalComparer ();
+
+			public int Compare (object a, object b)
+			{
+				return String.CompareOrdinal ((string) a, (string) b);
+			}
 		}
 	}
 }
