@@ -37,8 +37,6 @@ namespace Beagle.Daemon.KonqQueryable {
 	[QueryableFlavor (Name="KonquerorHistory", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class KonqQueryable : LuceneFileQueryable, IIndexableGenerator {
 
-		private static Logger log = Logger.Get ("KonqQueryable");
-
 		string konq_cache_dir;
 		private IEnumerator directory_enumerator = null;
 		private int polling_interval_in_seconds = 300; // 5 min
@@ -46,7 +44,10 @@ namespace Beagle.Daemon.KonqQueryable {
 		// ISO-Latin1 is 28591
 		private Encoding latin_encoding = Encoding.GetEncoding (28591);
 
-		public KonqQueryable () : base ("KonqHistoryIndex")
+		// Store URLs in the right form
+		const int MINOR_VERSION = 1;
+
+		public KonqQueryable () : base ("KonqHistoryIndex", MINOR_VERSION)
 		{
 			/* How to determine kio-http cache location ?
 			 * From KDE web-page it looks like /var/tmp/kdecache-$USERNAME/http
@@ -59,7 +60,7 @@ namespace Beagle.Daemon.KonqQueryable {
 
 			konq_cache_dir = Path.Combine (tmpdir, "kdecache-" + Environment.UserName ); 
 			konq_cache_dir = Path.Combine (konq_cache_dir, "http"); 
-			log.Debug ("KonqCacheDir: " + konq_cache_dir);
+			Log.Debug ("KonqCacheDir: " + konq_cache_dir);
 		}
 
 		/////////////////////////////////////////////////
@@ -90,18 +91,16 @@ namespace Beagle.Daemon.KonqQueryable {
 				ThisScheduler.Add (crawl_task);
 			}
 
-                        log.Info ("Starting Konq history backend ...");
+                        Log.Info ("Starting Konq history backend ...");
 			Crawl ();
 		}
 
 		private void Crawl ()
 		{
-                        State = QueryableState.Crawling;
 			directory_enumerator = DirectoryWalker.GetDirectoryInfos (konq_cache_dir).GetEnumerator ();
 			Scheduler.Task crawl_task = NewAddTask (this);
 			crawl_task.Tag = crawler_tag;
 			ThisScheduler.Add (crawl_task);
-			State = QueryableState.Idle;
 		}
 
 		private string crawler_tag = "Konqueror History Crawler";
@@ -152,7 +151,7 @@ namespace Beagle.Daemon.KonqQueryable {
 		{
 			if (path.EndsWith (".new"))
 				return;
-			Indexable indexable = FileToIndexable (path);
+			Indexable indexable = FileToIndexable (path, false);
 			if (indexable == null)
 				return;
 			Scheduler.Task task = NewAddTask (indexable);
@@ -164,7 +163,7 @@ namespace Beagle.Daemon.KonqQueryable {
 
 		/////////////////////////////////////////////////
 		
-		private Indexable FileToIndexable (string path) {
+		private Indexable FileToIndexable (string path, bool crawling) {
 			//Logger.Log.Debug ("KonqQ: Trying to index " + path);
 
 			FileStream stream;
@@ -194,7 +193,7 @@ namespace Beagle.Daemon.KonqQueryable {
 					return null; // we wont index bad files and non-html files
 				}
 
-				Logger.Log.Debug ("KonqQ: Indexing " + path + " with url=" + url);
+				//Logger.Log.Debug ("KonqQ: Indexing " + path + " with url=" + url);
 				Uri uri = new Uri (url, true);
 				if (uri.Scheme == Uri.UriSchemeHttps) {
 					Logger.Log.Error ("Indexing secure https:// URIs is not secure!");
@@ -208,8 +207,9 @@ namespace Beagle.Daemon.KonqQueryable {
 				indexable.AddProperty (Property.NewUnstored ("fixme:urltoken", StringFu.UrlFuzzyDivide (url)));
 				// hint for the filter about the charset
 				indexable.AddProperty (Property.NewUnsearched (StringFu.UnindexedNamespace + "charset", charset));
+				indexable.FlushBufferCache = crawling;
 			
-				DateTime date = new DateTime (1970, 1, 1);
+				DateTime date = DateTimeUtil.UnixToDateTimeUtc (0);
 				date = date.AddSeconds (Int64.Parse (creation_date));
 				indexable.Timestamp = date;
 
@@ -228,7 +228,7 @@ namespace Beagle.Daemon.KonqQueryable {
 		{
 			if (current_file == null)
 				return null;
-			return FileToIndexable (current_file.FullName);
+			return FileToIndexable (current_file.FullName, true);
 		}
 		
 		public bool HasNextIndexable ()
@@ -265,5 +265,22 @@ namespace Beagle.Daemon.KonqQueryable {
 		public void PostFlushHook ()
 		{ }
 
+		protected override QueryPart QueryPartHook (QueryPart part)
+		{
+			if (part is QueryPart_Property) {
+				QueryPart_Property prop_part = (QueryPart_Property) part;
+				if (prop_part.Key == "inuri") { // special case
+					QueryPart_Property new_part = new QueryPart_Property ();
+					new_part.Logic = prop_part.Logic;
+					new_part.Key = "fixme:urltoken";
+					new_part.Type = PropertyType.Text;
+					new_part.Value = prop_part.Value;
+
+					return new_part;
+				}
+			}
+
+			return part;
+		}
 	}
 }
