@@ -1,7 +1,7 @@
 //
 // LuceneCommon.cs
 //
-// Copyright (C) 2004-2005 Novell, Inc.
+// Copyright (C) 2004-2007 Novell, Inc.
 //
 
 //
@@ -26,6 +26,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.Diagnostics;
 using System.Globalization;
 using System.IO;
@@ -35,12 +36,16 @@ using System.Xml;
 using System.Xml.Serialization;
 
 using Lucene.Net.Analysis;
+using Lucene.Net.Analysis.Snowball;
 using Lucene.Net.Analysis.Standard;
 using Lucene.Net.Documents;
 using Lucene.Net.Index;
 using Lucene.Net.QueryParsers;
 using LNS = Lucene.Net.Search;
 
+using SF.Snowball.Ext;
+using SnowballProgram = SF.Snowball.SnowballProgram;
+
 using Beagle.Util;
 
 namespace Beagle.Daemon {
@@ -48,6 +53,7 @@ namespace Beagle.Daemon {
 	public class LuceneCommon {
 
 		public delegate bool HitFilter (Hit hit);
+		public delegate QueryPart QueryPartHook (QueryPart query_part);
 
 		// VERSION HISTORY
 		// ---------------
@@ -78,7 +84,11 @@ namespace Beagle.Daemon {
 		//     lower case so that we're truly case insensitive.
 		// 16: add inverted timestamp to make querying substantially faster
 		// 17: add boolean property to denote a child indexable
-		private const int MAJOR_VERSION = 17;
+		// 18: add IsPersistent to properties, and adjust coded values
+		//     in AddPropertyToDocument() and GetPropertyFromDocument();
+		//     changed subdate field format rules for better readability
+		// 19: Update lucene and analyzer (lucene.net-2.1)
+		private const int MAJOR_VERSION = 19;
 		private int minor_version = 0;
 
 		private string index_name;
@@ -95,6 +105,9 @@ namespace Beagle.Daemon {
 		// expect to have change.  Canonical example: file names.
 		private Lucene.Net.Store.Directory secondary_store = null;
 
+		// Flush if more than this number of requests
+		public const int RequestFlushThreshold = Lucene.Net.Index.IndexWriter.DEFAULT_MAX_BUFFERED_DOCS; // Use same value as Lucene's flush threshold
+
 		//////////////////////////////////////////////////////////////////////////////
 
 		protected LuceneCommon (string index_name, int minor_version)
@@ -246,12 +259,17 @@ namespace Beagle.Daemon {
 			int current_major_version, current_minor_version;
 			int i = version_str.IndexOf ('.');
 			
-			if (i != -1) {
-				current_major_version = Convert.ToInt32 (version_str.Substring (0, i));
-				current_minor_version = Convert.ToInt32 (version_str.Substring (i+1));
-			} else {
-				current_minor_version = Convert.ToInt32 (version_str);
-				current_major_version = 0;
+			try {
+				if (i != -1) {
+					current_major_version = Convert.ToInt32 (version_str.Substring (0, i));
+					current_minor_version = Convert.ToInt32 (version_str.Substring (i+1));
+				} else {
+					current_minor_version = Convert.ToInt32 (version_str);
+					current_major_version = 0;
+				}
+			} catch (FormatException) {
+				// Something wrong with the version file.
+				return false;
 			}
 
 			if (current_major_version != MAJOR_VERSION
@@ -267,13 +285,98 @@ namespace Beagle.Daemon {
 			// assume that the index is corrupted and declare it non-existent.
 			DirectoryInfo lock_dir_info;
 			lock_dir_info = new DirectoryInfo (LockDirectory);
+			bool dangling_lock = false;
+
 			foreach (FileInfo info in lock_dir_info.GetFiles ()) {
 				if (IsDanglingLock (info)) {
-					Logger.Log.Warn ("Found a dangling index lock on {0}", info.FullName);
-					return false;
+					Logger.Log.Warn ("Found a dangling index lock on {0}.", info.FullName);
+					dangling_lock = true;
 				}
 			}
 
+			if (dangling_lock) {
+				Beagle.Util.Stopwatch w = new Beagle.Util.Stopwatch ();
+				w.Start ();
+
+				if (VerifyLuceneIndex (PrimaryIndexDirectory) && 
+				    VerifyLuceneIndex (SecondaryIndexDirectory)) {
+					w.Stop ();
+
+					Log.Warn ("Indexes verified in {0}.  Deleting stale lock files.", w);
+
+					try {
+						foreach (FileInfo info in lock_dir_info.GetFiles ())
+							info.Delete ();
+					} catch {
+						Log.Warn ("Could not delete lock files.");
+						return false;
+					}
+					return true;
+				} else
+					return false;
+			}
+
+			return true;
+		}
+
+		private bool VerifyLuceneIndex (string path)
+		{
+			if (! Directory.Exists (path))
+				return true;
+
+			Log.Debug ("Verifying index {0}", path);
+
+			IndexReader reader = null;
+			TermEnum enumerator = null;
+			TermPositions positions = null;
+
+			try {
+				reader = IndexReader.Open (path);
+
+				// Crawl all of the terms in the index, and get the
+				// term positions and crawl those as well.  This
+				// method is suggested as a way to verify the index
+				// here:
+				//
+				// http://mail-archives.apache.org/mod_mbox/lucene-java-user/200504.mbox/%3c4265767B.5090307@getopt.org%3e
+				enumerator = reader.Terms ();
+
+				do {
+					Term term = enumerator.Term ();
+					if (term == null)
+						break;
+
+					positions = reader.TermPositions (term);
+
+					while (positions.Next ()) {
+						int freq = positions.Freq ();
+
+						for (int i = 0; i < freq; i++)
+							positions.NextPosition ();
+					}
+					positions.Close ();
+					positions = null;
+				} while (enumerator.Next ());
+
+				enumerator.Close ();
+				enumerator = null;
+
+				reader.Close ();
+				reader = null;
+			} catch (Exception e) {
+				Log.Warn ("Lucene index {0} is corrupted ({1})", path, e.Message);
+				return false;
+			} finally {
+				if (positions != null)
+					positions.Close ();
+
+				if (enumerator != null)
+					enumerator.Close ();
+
+				if (reader != null)
+					reader.Close ();
+			}
+
 			return true;
 		}
 
@@ -284,7 +387,7 @@ namespace Beagle.Daemon {
 
 			// Create a new store.
 			Lucene.Net.Store.Directory store;
-			store = Lucene.Net.Store.FSDirectory.GetDirectory (path, LockDirectory, true);
+			store = Lucene.Net.Store.FSDirectory.GetDirectory (path, new Lucene.Net.Store.SimpleFSLockFactory (LockDirectory));
 
 			// Create an empty index in that store.
 			IndexWriter writer;
@@ -342,8 +445,14 @@ namespace Beagle.Daemon {
 			reader.Close ();
 
 			// Create stores for our indexes.
-			primary_store = Lucene.Net.Store.FSDirectory.GetDirectory (PrimaryIndexDirectory, LockDirectory, false, read_only_mode);
-			secondary_store = Lucene.Net.Store.FSDirectory.GetDirectory (SecondaryIndexDirectory, LockDirectory, false, read_only_mode);
+			// Use separate lock factories since each lock factory is tied to the index directory
+			if (read_only_mode) {
+				primary_store = Lucene.Net.Store.FSDirectory.GetDirectory (PrimaryIndexDirectory, Lucene.Net.Store.NoLockFactory.GetNoLockFactory ());
+				secondary_store = Lucene.Net.Store.FSDirectory.GetDirectory (SecondaryIndexDirectory, Lucene.Net.Store.NoLockFactory.GetNoLockFactory ());
+			} else {
+				primary_store = Lucene.Net.Store.FSDirectory.GetDirectory (PrimaryIndexDirectory, new Lucene.Net.Store.SimpleFSLockFactory (LockDirectory));
+				secondary_store = Lucene.Net.Store.FSDirectory.GetDirectory (SecondaryIndexDirectory, new Lucene.Net.Store.SimpleFSLockFactory (LockDirectory));
+			}
 		}
 
 		////////////////////////////////////////////////////////////////
@@ -376,8 +485,9 @@ namespace Beagle.Daemon {
 		}
 
 		// FIXME: This assumes everything being indexed is in English!
-		internal class BeagleAnalyzer : StandardAnalyzer {
+		public class BeagleAnalyzer : StandardAnalyzer {
 
+			const string DEFAULT_STEMMER_LANGUAGE = "English";
 			private char [] buffer = new char [2];
 			private bool strip_extra_property_info = false;
 			private bool tokenize_email_hostname = false;
@@ -437,7 +547,10 @@ namespace Beagle.Daemon {
 				    || fieldName == "PropertyText"
 				    || is_text_prop) {
 					outstream = new NoiseEmailHostFilter (outstream, tokenize_email_hostname);
-					outstream = new PorterStemFilter (outstream);
+					// Sharing Stemmer is not thread safe.
+					// Currently our underlying lucene indexing is not done in multiple threads.
+					StemmerInfo stemmer_info = GetStemmer (DEFAULT_STEMMER_LANGUAGE);
+					outstream = new SnowballFilter (outstream, stemmer_info.Stemmer, stemmer_info.StemMethod);
 				}
 
 				return outstream;
@@ -511,13 +624,13 @@ namespace Beagle.Daemon {
 			DateTime dt = StringFu.StringToDateTime (prop.Value);
 
 			Field f;
-			f = new Field ("YM:" + field_name,
+			f = new Field (field_name + "(YM)",
 				       StringFu.DateTimeToYearMonthString (dt),
 				       Field.Store.NO,
 				       Field.Index.NO_NORMS);
 			doc.Add (f);
 
-			f = new Field ("D:" + field_name,
+			f = new Field (field_name + "(D)",
 				       StringFu.DateTimeToDayString (dt),
 				       Field.Store.NO,
 				       Field.Index.NO_NORMS);
@@ -526,7 +639,7 @@ namespace Beagle.Daemon {
 
 		static protected void AddPropertyToDocument (Property prop, Document doc)
 		{
-			if (prop == null || prop.Value == null)
+			if (prop == null || String.IsNullOrEmpty (prop.Value))
 				return;
 
 			// Don't actually put properties in the UnindexedNamespace
@@ -556,8 +669,9 @@ namespace Beagle.Daemon {
 			}
 
 			string coded_value;
-			coded_value = String.Format ("{0}:{1}",
+			coded_value = String.Format ("{0}{1}:{2}",
 						     prop.IsSearched ? 's' : '_',
+						     prop.IsPersistent ? 'p' : '_',
 						     prop.Value);
 
 			string field_name = PropertyToFieldName (prop.Type, prop.Key);
@@ -572,6 +686,8 @@ namespace Beagle.Daemon {
 				AddDateFields (field_name, prop, doc);
 		}
 
+		public static bool DumpIndexMode = false;
+
 		static protected Property GetPropertyFromDocument (Field f, Document doc, bool from_primary_index)
 		{
 			// Note: we don't use the document that we pass in,
@@ -582,23 +698,38 @@ namespace Beagle.Daemon {
 			if (f == null)
 				return null;
 
+			bool internal_prop = false;
+
 			string field_name;
 			field_name = f.Name ();
 			if (field_name.Length < 7
-			    || ! field_name.StartsWith ("prop:"))
-				return null;
+			    || ! field_name.StartsWith ("prop:")) {
+				if (DumpIndexMode)
+					internal_prop = true;
+				else
+					return null;
+			}
 
 			string field_value;
 			field_value = f.StringValue ();
 
 			Property prop;
 			prop = new Property ();
-			prop.Type = CodeToType (field_name [5]);
-			prop.Key = field_name.Substring (7);
-			prop.Value = field_value.Substring (2);
+
+			if (DumpIndexMode) {
+				prop.Type = CodeToType ( internal_prop ? 'k' : field_name [5]);
+				prop.Key = (internal_prop ? field_name : field_name.Substring (7));
+				prop.Value = (internal_prop ? field_value : field_value.Substring (3));
+			} else {
+				prop.Type = CodeToType (field_name [5]);
+				prop.Key = field_name.Substring (7);
+				prop.Value = field_value.Substring (3);
+			}
+
 			prop.IsSearched = (field_value [0] == 's');
+			prop.IsPersistent = (field_value [1] == 'p');
 			prop.IsMutable = ! from_primary_index;
-			prop.IsStored = f.IsStored ();
+			prop.IsStored = true; // Unstored fields cannot be retrieved
 
 			return prop;
 		}
@@ -618,6 +749,11 @@ namespace Beagle.Daemon {
 
 			Field f;
 
+			// During querying, we retrieve a lucene document with only certain fields
+			// like Uri and Timestamp and quickly check if the document is a good one
+			// The field specified document constructor runs faster if the fields that
+			// are asked for are located at the beginning of the document.
+			// Hence it is better to keep "Uri" and "Timestamp" at the beginning.
 			f = new Field ("Uri", UriFu.UriToEscapedString (indexable.Uri),
 				       Field.Store.YES, Field.Index.NO_NORMS);
 			primary_doc.Add (f);
@@ -645,21 +781,22 @@ namespace Beagle.Daemon {
 				// Create an inverted timestamp so that we can
 				// sort by timestamp at search-time.
 				long timeval = Convert.ToInt64 (str);
-				f = new Field ("InvertedTimestamp", (Int64.MaxValue - timeval).ToString (),
+				// Pad the inverted timestamp with zeroes for proper string comparison during termenum enumeration
+				f = new Field ("InvertedTimestamp", (Int64.MaxValue - timeval).ToString ("d19"),
 					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				str = StringFu.DateTimeToYearMonthString (indexable.Timestamp);
-				f = new Field ("YM:Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
+				f = new Field ("Timestamp(YM)", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = new Field ("YM:" + wildcard_field, str,
+				f = new Field (wildcard_field + "(YM)", str,
 					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 
 				str = StringFu.DateTimeToDayString (indexable.Timestamp);
-				f = new Field ("D:Timestamp", str, Field.Store.YES, Field.Index.NO_NORMS);
+				f = new Field ("Timestamp(D)", str, Field.Store.YES, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
-				f = new Field ("D:" + wildcard_field, str,
+				f = new Field (wildcard_field + "(D)", str,
 					       Field.Store.NO, Field.Index.NO_NORMS);
 				primary_doc.Add (f);
 			}
@@ -686,12 +823,13 @@ namespace Beagle.Daemon {
 					primary_doc.Add (f);
 				}
 			
+				// FIXME: HotText is ignored for now!
 				// Then add "HotText"
-				reader = indexable.GetHotTextReader ();
-				if (reader != null) {
-					f = new Field ("HotText", reader);
-					primary_doc.Add (f);
-				}
+				//reader = indexable.GetHotTextReader ();
+				//if (reader != null) {
+				//	f = new Field ("HotText", reader);
+				//	primary_doc.Add (f);
+				//}
 			}
 
 			// Store the Type and MimeType in special properties
@@ -779,6 +917,14 @@ namespace Beagle.Daemon {
 			// return w/o doing anything.
 			foreach (Property prop in prop_only_indexable.Properties) {
 				seen_props [prop.Key] = prop;
+
+				// Don't add properties that are empty; they
+				// essentially mean "reset this property"
+				if (prop.Value == String.Empty) {
+					Log.Debug ("Resetting prop '{0}'", prop.Key);
+					continue;
+				}
+
 				AddPropertyToDocument (prop, new_doc);
 				Logger.Log.Debug ("New prop '{0}' = '{1}'", prop.Key, prop.Value);
 			}
@@ -800,6 +946,27 @@ namespace Beagle.Daemon {
 			return new_doc;
 		}
 
+		static protected Document MergeDocuments (Document doc,
+							  Document prop_change_doc)
+		{
+			if (doc == null)
+				return prop_change_doc;
+
+			if (prop_change_doc == null)
+				return doc;
+
+			foreach (Field f in prop_change_doc.Fields ()) {
+				Property prop = GetPropertyFromDocument (f, prop_change_doc, false);
+
+				if (prop != null && prop.IsPersistent) {
+					Log.Debug ("Moving old persistent prop '{0}' = '{1}' forward", prop.Key, prop.Value);
+					AddPropertyToDocument (prop, doc);
+				}
+			}
+
+			return doc;
+		}
+
 		static protected Uri GetUriFromDocument (Document doc)
 		{
 			string uri;
@@ -825,11 +992,6 @@ namespace Beagle.Daemon {
 
 			AddPropertiesToHit (hit, doc, true);
 
-			// Get the Type and MimeType from the properties.
-			hit.Type = hit.GetFirstProperty ("beagle:HitType");
-			hit.MimeType = hit.GetFirstProperty ("beagle:MimeType");
-			hit.Source = hit.GetFirstProperty ("beagle:Source");
-
 			return hit;
 		}
 
@@ -888,11 +1050,45 @@ namespace Beagle.Daemon {
 		// Access to the stemmer and list of stop words
 		//
 
-		static PorterStemmer stemmer = new PorterStemmer ();
+		private static Dictionary<string, StemmerInfo> stemmer_table = new Dictionary<string, StemmerInfo> ();
+
+		class StemmerInfo {
+			internal SnowballProgram Stemmer;
+			internal System.Reflection.MethodInfo StemMethod;
+		}
+
+		private static StemmerInfo GetStemmer (System.String name)
+		{
+			if (! stemmer_table.ContainsKey (name)) {
+				StemmerInfo stemmer_info = new StemmerInfo ();
+
+				// Taken from Snowball/SnowballFilter.cs
+				System.Type stemClass = System.Type.GetType ("SF.Snowball.Ext." + name + "Stemmer", true);
+				SnowballProgram stemmer = (SnowballProgram) System.Activator.CreateInstance (stemClass);
+				// why doesn't the SnowballProgram class have an (abstract?) stem method?
+				System.Reflection.MethodInfo stemMethod = stemClass.GetMethod ("Stem", (new System.Type [0] == null) ? new System.Type [0] : (System.Type []) new System.Type [0]);
+
+				stemmer_info.Stemmer = stemmer;
+				stemmer_info.StemMethod = stemMethod;
+				stemmer_table [name] = stemmer_info;
+			}
+
+			return stemmer_table [name];
+		}
+
+		private static SF.Snowball.Ext.EnglishStemmer default_stemmer = new SF.Snowball.Ext.EnglishStemmer ();
 
 		static public string Stem (string str)
 		{
-			return stemmer.Stem (str);
+			string stemmed_str;
+
+			lock (default_stemmer) {
+				default_stemmer.SetCurrent (str);
+				default_stemmer.Stem ();
+				stemmed_str = default_stemmer.GetCurrent ();
+			}
+
+			return stemmed_str;
 		}
 
 		public static bool IsStopWord (string stemmed_word)
@@ -974,7 +1170,7 @@ namespace Beagle.Daemon {
 		// Queries
 		//
 
-		static private LNS.Query StringToQuery (string field_name,
+		static internal LNS.Query StringToQuery (string field_name,
 							string text,
 							ArrayList term_list)
 		{
@@ -1041,7 +1237,7 @@ namespace Beagle.Daemon {
 
 		static private Term NewYearMonthTerm (string field_name, int y, int m)
 		{
-			return new Term ("YM:" + field_name, String.Format ("{0}{1:00}", y, m));
+			return new Term (field_name + "(YM)", String.Format ("{0}{1:00}", y, m));
 		}
 
 		static private LNS.Query NewYearMonthQuery (string field_name, int y, int m)
@@ -1058,11 +1254,14 @@ namespace Beagle.Daemon {
 
 		static private Term NewDayTerm (string field_name, int d)
 		{
-			return new Term ("D:" + field_name, String.Format ("{0:00}", d));
+			return new Term (field_name + "(D)", String.Format ("{0:00}", d));
 		}
 
 		static private LNS.Query NewDayQuery (string field_name, int d1, int d2)
 		{
+			if (d1 == d2)
+				return new LNS.TermQuery (NewDayTerm (field_name, d1));
+
 			return new LNS.RangeQuery (NewDayTerm (field_name, d1),
 						   NewDayTerm (field_name, d2),
 						   true); // query is inclusive
@@ -1175,11 +1374,11 @@ namespace Beagle.Daemon {
 				if (d1 != 1 || d2 != DateTime.DaysInMonth (y2, m2)) {
 					LNS.BooleanQuery sub_query;
 					sub_query = new LNS.BooleanQuery ();
-					sub_query.Add (ym_query, true, false);
-					sub_query.Add (NewDayQuery (field_name, d1, d2), true, false);
-					top_level_query.Add (sub_query, false, false);
+					sub_query.Add (ym_query, LNS.BooleanClause.Occur.MUST);
+					sub_query.Add (NewDayQuery (field_name, d1, d2), LNS.BooleanClause.Occur.MUST);
+					top_level_query.Add (sub_query, LNS.BooleanClause.Occur.SHOULD);
 				} else {
-					top_level_query.Add (ym_query, false, false);
+					top_level_query.Add (ym_query, LNS.BooleanClause.Occur.SHOULD);
 				}
 
 			} else {
@@ -1188,9 +1387,9 @@ namespace Beagle.Daemon {
 				if (d1 > 1) {
 					LNS.BooleanQuery sub_query;
 					sub_query = new LNS.BooleanQuery ();
-					sub_query.Add (NewYearMonthQuery (field_name, y1, m1), true, false);
-					sub_query.Add (NewDayQuery (field_name, d1, DateTime.DaysInMonth (y1, m1)), true, false);
-					top_level_query.Add (sub_query, false, false);
+					sub_query.Add (NewYearMonthQuery (field_name, y1, m1), LNS.BooleanClause.Occur.MUST);
+					sub_query.Add (NewDayQuery (field_name, d1, DateTime.DaysInMonth (y1, m1)), LNS.BooleanClause.Occur.MUST);
+					top_level_query.Add (sub_query, LNS.BooleanClause.Occur.SHOULD);
 					
 					++m1;
 					if (m1 == 13) {
@@ -1203,9 +1402,9 @@ namespace Beagle.Daemon {
 				if (d2 < DateTime.DaysInMonth (y2, m2)) {
 					LNS.BooleanQuery sub_query;
 					sub_query = new LNS.BooleanQuery ();
-					sub_query.Add (NewYearMonthQuery (field_name, y2, m2), true, false);
-					sub_query.Add (NewDayQuery (field_name, 1, d2), true, false);
-					top_level_query.Add (sub_query, false, false);
+					sub_query.Add (NewYearMonthQuery (field_name, y2, m2), LNS.BooleanClause.Occur.MUST);
+					sub_query.Add (NewDayQuery (field_name, 1, d2), LNS.BooleanClause.Occur.MUST);
+					top_level_query.Add (sub_query, LNS.BooleanClause.Occur.SHOULD);
 
 					--m2;
 					if (m2 == 0) {
@@ -1217,7 +1416,7 @@ namespace Beagle.Daemon {
 				// Generate the query for the "middle" of our period, if it is non-empty
 				if (y1 < y2 || ((y1 == y2) && m1 <= m2))
 					top_level_query.Add (NewYearMonthQuery (field_name, y1, m1, y2, m2),
-							     false, false);
+							     LNS.BooleanClause.Occur.SHOULD);
 			}
 				
 			return top_level_query;
@@ -1228,6 +1427,7 @@ namespace Beagle.Daemon {
 		static protected void QueryPartToQuery (QueryPart     abstract_part,
 							bool          only_build_primary_query,
 							ArrayList     term_list,
+							QueryPartHook query_part_hook,
 							out LNS.Query primary_query,
 							out LNS.Query secondary_query,
 							out HitFilter hit_filter)
@@ -1251,6 +1451,14 @@ namespace Beagle.Daemon {
 			if (abstract_part == null)
 				return;
 
+			// Run the backend hook first.
+			// This gives a chance to modify create new queries based on
+			// backend specific properties
+
+			abstract_part = query_part_hook (abstract_part);
+			if (abstract_part == null)
+				return;
+
 			if (abstract_part is QueryPart_Text) {
 				QueryPart_Text part = (QueryPart_Text) abstract_part;
 
@@ -1266,14 +1474,14 @@ namespace Beagle.Daemon {
 					LNS.Query subquery;
 					subquery = StringToQuery ("Text", part.Text, term_list);
 					if (subquery != null) {
-						p_query.Add (subquery, false, false);
+						p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 						added_subquery = true;
 					}
 
 					// FIXME: HotText is ignored for now!
 					// subquery = StringToQuery ("HotText", part.Text);
 					// if (subquery != null) {
-					//    p_query.Add (subquery, false, false);
+					//    p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 					//    added_subquery = true;
 					// }
 				}
@@ -1282,10 +1490,10 @@ namespace Beagle.Daemon {
 					LNS.Query subquery;
 					subquery = StringToQuery ("PropertyText", part.Text, term_list);
 					if (subquery != null) {
-						p_query.Add (subquery, false, false);
+						p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 						// Properties can live in either index
 						if (! only_build_primary_query)
-							s_query.Add (subquery.Clone () as LNS.Query, false, false);
+							s_query.Add (subquery.Clone () as LNS.Query, LNS.BooleanClause.Occur.SHOULD);
 						added_subquery = true;
 					}
 
@@ -1316,10 +1524,10 @@ namespace Beagle.Daemon {
 						if (term_list != null)
 							term_list.Add (term);
 						subquery = new LNS.TermQuery (term);
-						p_query.Add (subquery, false, false);
+						p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 						// Properties can live in either index
 						if (! only_build_primary_query)
-							s_query.Add (subquery.Clone () as LNS.Query, false, false);
+							s_query.Add (subquery.Clone () as LNS.Query, LNS.BooleanClause.Occur.SHOULD);
 					} else {
 						// Reset these so we return a null query
 						p_query = null;
@@ -1349,26 +1557,26 @@ namespace Beagle.Daemon {
 				// Search text content
 				term = new Term ("Text", query_string_lower);
 				subquery = new LNS.WildcardQuery (term);
-				p_query.Add (subquery, false, false);
+				p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 				term_list.Add (term);
 
 				// Search text properties
 				term = new Term ("PropertyText", query_string_lower);
 				subquery = new LNS.WildcardQuery (term);
-				p_query.Add (subquery, false, false);
+				p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 				// Properties can live in either index
 				if (! only_build_primary_query)
-					s_query.Add (subquery.Clone () as LNS.Query, false, false);
+					s_query.Add (subquery.Clone () as LNS.Query, LNS.BooleanClause.Occur.SHOULD);
 				term_list.Add (term);
 
 				// Search property keywords
 				term = new Term ("PropertyKeyword", query_string_lower);
 				term_list.Add (term);
 				subquery = new LNS.WildcardQuery (term);
-				p_query.Add (subquery, false, false);
+				p_query.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 				// Properties can live in either index
 				if (! only_build_primary_query)
-					s_query.Add (subquery.Clone () as LNS.Query, false, false);
+					s_query.Add (subquery.Clone () as LNS.Query, LNS.BooleanClause.Occur.SHOULD);
 
 				primary_query = p_query;
 				if (! only_build_primary_query)
@@ -1377,47 +1585,20 @@ namespace Beagle.Daemon {
 				return;
 			}
 
-			if (abstract_part is QueryPart_Property) {
-				QueryPart_Property part = (QueryPart_Property) abstract_part;
-
-				string field_name;
-				if (part.Key == QueryPart_Property.AllProperties)
-					field_name = TypeToWildcardField (part.Type);
-				else
-					field_name = PropertyToFieldName (part.Type, part.Key);
-
-				if (part.Type == PropertyType.Text)
-					primary_query = StringToQuery (field_name, part.Value, term_list);
-				else {
-					Term term;
-					term = new Term (field_name, part.Value.ToLower ());
-					if (term_list != null)
-						term_list.Add (term);
-					primary_query = new LNS.TermQuery (term);
-				}
-
-				// Properties can live in either index
-				if (! only_build_primary_query && primary_query != null)
-					secondary_query = primary_query.Clone () as LNS.Query;
-
-				return;
-			}
-
 			if (abstract_part is QueryPart_DateRange) {
 
 				QueryPart_DateRange part = (QueryPart_DateRange) abstract_part;
 
+				// FIXME: We don't handle prohibited queries with sub-date
+				// accuracy.  For example, if we say we prohibit matches
+				// between 5 May 2007 at 2 PM and 8 May at 5 AM, we'll
+				// miss any matches that happen between midnight and 2 PM
+				// on 5 May 2007 and between midnight and 5 AM on 8 May.
+
 				primary_query = GetDateRangeQuery (part, out hit_filter);
 				// Date properties can live in either index
 				if (! only_build_primary_query && primary_query != null)
 					secondary_query = primary_query.Clone () as LNS.Query;
-
-				// If this is a prohibited part, invert our hit filter.
-				if (part.Logic == QueryPartLogic.Prohibited) {
-					NotHitFilter nhf;
-					nhf = new NotHitFilter (hit_filter);
-					hit_filter = new HitFilter (nhf.HitFilter);
-				}
 				
 				return;
 			}
@@ -1445,12 +1626,12 @@ namespace Beagle.Daemon {
 					// correctly, because we can't tell which part of an OR we matched
 					// against to filter correctly.  This affects date range queries.
 					QueryPartToQuery (sub_part, only_build_primary_query,
-							  term_list,
+							  term_list, query_part_hook,
 							  out p_subq, out s_subq, out sub_hit_filter);
 					if (p_subq != null)
-						p_query.Add (p_subq, false, false);
+						p_query.Add (p_subq, LNS.BooleanClause.Occur.SHOULD);
 					if (s_subq != null)
-						s_query.Add (s_subq, false, false);
+						s_query.Add (s_subq, LNS.BooleanClause.Occur.SHOULD);
 					if (sub_hit_filter != null) {
 						if (or_hit_filter == null)
 							or_hit_filter = new OrHitFilter ();
@@ -1464,6 +1645,52 @@ namespace Beagle.Daemon {
 				return;
 			} 
 
+			if (abstract_part is QueryPart_Uri) {
+				QueryPart_Uri part = (QueryPart_Uri) abstract_part;
+
+				// Do a term query on the Uri field.
+				// This is probably less efficient that using a TermEnum;
+				// but this is required for the query API where the uri query
+				// can be part of a prohibited query or a boolean or query.
+				Term term;
+				term = new Term ("Uri", UriFu.UriToEscapedString (part.Uri));
+				if (term_list != null)
+					term_list.Add (term);
+				primary_query = new LNS.TermQuery (term);
+
+				// Query only the primary index
+				return;
+			}
+
+			if (abstract_part is QueryPart_Property) {
+				QueryPart_Property part = (QueryPart_Property) abstract_part;
+
+				string field_name;
+				if (part.Key == QueryPart_Property.AllProperties)
+					field_name = TypeToWildcardField (part.Type);
+				else
+					field_name = PropertyToFieldName (part.Type, part.Key);
+
+				if (part.Type == PropertyType.Text)
+					primary_query = StringToQuery (field_name, part.Value, term_list);
+				else {
+					Term term;
+					if (field_name.StartsWith ("prop:k:" + Property.PrivateNamespace))
+						term = new Term (field_name, part.Value);
+					else
+						term = new Term (field_name, part.Value.ToLower ());
+					if (term_list != null)
+						term_list.Add (term);
+					primary_query = new LNS.TermQuery (term);
+				}
+
+				// Properties can live in either index
+				if (! only_build_primary_query && primary_query != null)
+					secondary_query = primary_query.Clone () as LNS.Query;
+
+				return;
+			}
+
 			throw new Exception ("Unhandled QueryPart type! " + abstract_part.ToString ());
 		}
 
@@ -1493,7 +1720,7 @@ namespace Beagle.Daemon {
 
 			int cursor = 0;
 			if (extra_requirement != null) {
-				top_query.Add (extra_requirement, true, false);
+				top_query.Add (extra_requirement, LNS.BooleanClause.Occur.MUST);
 				++cursor;
 			}
 
@@ -1505,7 +1732,7 @@ namespace Beagle.Daemon {
 					LNS.BooleanQuery bq;
 					bq = new LNS.BooleanQuery ();
 					bottom_queries.Add (bq);
-					top_query.Add (bq, false, false);
+					top_query.Add (bq, LNS.BooleanClause.Occur.SHOULD);
 				}
 			}
 
@@ -1523,7 +1750,7 @@ namespace Beagle.Daemon {
 						cursor = 0;
 				}
 				
-				target.Add (subquery, false, false);
+				target.Add (subquery, LNS.BooleanClause.Occur.SHOULD);
 			}
 
 			return top_query;
