@@ -32,10 +32,11 @@ using System.Reflection;
 using System.Text;
 using System.Threading;
 using Beagle.Util;
+using System.Xml.Serialization;
 
 namespace Beagle.Daemon {
 	
-	public class QueryDriver {
+	public static class QueryDriver {
 
 		// Contains list of queryables explicitly asked by --allow-backend or --backend name
 		// --allow-backend/--backend name : dont read config information and only start backend 'name'
@@ -129,6 +130,7 @@ namespace Beagle.Daemon {
 
 		public static int IndexingDelay {
 			set { indexing_delay = value; }
+			get { return indexing_delay; }
 		}
 
 		//////////////////////////////////////////////////////////////////////////////////////
@@ -149,7 +151,6 @@ namespace Beagle.Daemon {
 			int count = 0;
 
 			foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (IQueryableTypesAttribute))) {
-				bool type_accepted = false;
 				foreach (QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (QueryableFlavor))) {
 					if (! UseQueryable (flavor.Name))
 						continue;
@@ -175,59 +176,15 @@ namespace Beagle.Daemon {
 						Queryable q = new Queryable (flavor, iq);
 						iqueryable_to_queryable [iq] = q;
 						++count;
-						type_accepted = true;
 						break;
 					}
 				}
-
-				if (! type_accepted)
-					continue;
-
-				object[] attributes = type.GetCustomAttributes (false);
-				foreach (object attribute in attributes) {
-					PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
-					if (mapping == null)
-						continue;
-					//Logger.Log.Debug (mapping.Keyword + " => " 
-					//		+ mapping.PropertyName + 
-					//		+ " is-keyword=" + mapping.IsKeyword + " (" 
-					//		+ mapping.Description + ") "
-					//		+ "(" + type.FullName + ")");
-					PropertyKeywordFu.RegisterMapping (mapping);
-				}
-					
 			}
 			Logger.Log.Debug ("Found {0} backends in {1}", count, assembly.Location);
 		}
 
 		////////////////////////////////////////////////////////
 
-		public static void ReadKeywordMappings ()
-		{
-			Logger.Log.Debug ("Reading mapping from filters");
-			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_FILTER_PATH", PathFinder.FilterDir);
-
-			foreach (Assembly assembly in assemblies) {
-				foreach (Type type in ReflectionFu.GetTypesFromAssemblyAttribute (assembly, typeof (FilterTypesAttribute))) {
-					object[] attributes = type.GetCustomAttributes (false);
-					foreach (object attribute in attributes) {
-						
-						PropertyKeywordMapping mapping = attribute as PropertyKeywordMapping;
-						if (mapping == null)
-							continue;
-						//Logger.Log.Debug (mapping.Keyword + " => " 
-						//		+ mapping.PropertyName
-						//		+ " is-keyword=" + mapping.IsKeyword + " (" 
-						//		+ mapping.Description + ") "
-						//		+ "(" + type.FullName + ")");
-						PropertyKeywordFu.RegisterMapping (mapping);
-					}
-				}
-			}
-		}
-
-		////////////////////////////////////////////////////////
-
 		// Scans PathFinder.SystemIndexesDir after available 
 		// system-wide indexes.
 		static void LoadSystemIndexes () 
@@ -362,7 +319,7 @@ namespace Beagle.Daemon {
 			
 			assemblies = null;
 
-			ReadKeywordMappings ();
+			PropertyKeywordFu.ReadKeywordMappings ();
 
 			LoadSystemIndexes ();
 			LoadStaticQueryables ();
@@ -577,7 +534,7 @@ namespace Beagle.Daemon {
 					continue;
 				if (sb.Length > 0)
 					sb.Append (' ');
-				sb.Append (LuceneCommon.Stem (split [i]));
+				sb.Append (LuceneCommon.Stem (split [i].ToLower ()));
 			}
 			response.StemmedText.Add (sb.ToString ());
 		}
@@ -674,6 +631,24 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
+		static public int DoCountMatchQuery (CountMatchQuery query)
+		{
+			DehumanizeQuery (query);
+
+			int num_matches = 0;
+
+			foreach (Queryable q in Queryables) {
+				if (! q.AcceptQuery (query))
+					continue;
+
+				num_matches += q.DoCountMatchQuery (query);
+			}
+
+			return num_matches;
+		}
+
+		////////////////////////////////////////////////////////
+
 		static public IEnumerable GetIndexInformation ()
 		{
 			foreach (Queryable q in Queryables)
@@ -703,5 +678,25 @@ namespace Beagle.Daemon {
 				return false;
 			}
 		}
+
+		/////////////////////////////////////////////////////////
+
+		// Various debug hooks; especially persistant state information and memory issues
+
+		public static void DebugHook ()
+		{
+			Log.Debug ("Debughook called:");
+			LuceneQueryable l;
+			foreach (Queryable q in Queryables) {
+				l = q.IQueryable as LuceneQueryable;
+				if (l == null)
+					continue;
+				try {
+					l.DebugHook ();
+				} catch (Exception e) {
+					Log.Debug (e, "Exception during Debughook for {0}", q.Name);
+				}
+			}
+		}
 	}
 }
