@@ -36,13 +36,10 @@ using Beagle.Util;
 
 namespace Beagle.Daemon {
 
-	[PropertyKeywordMapping (Keyword="extension", PropertyName="beagle:FilenameExtension", IsKeyword=true, Description="File extension, e.g. extension:jpeg. Use extension: to search in files with no extension.")]
-	[PropertyKeywordMapping (Keyword="ext", PropertyName="beagle:FilenameExtension", IsKeyword=true, Description="File extension, e.g. ext:jpeg. Use ext: to search in files with no extension.")]
-
 	public class StaticQueryable : LuceneQueryable 	{
-		
+
 		protected TextCache text_cache;
-		
+
 		public StaticQueryable (string index_name, string index_path, bool read_only_mode) : base (index_path, read_only_mode)
 		{
 			Logger.Log.Debug ("Initializing static queryable: {0}", index_path);
@@ -56,7 +53,7 @@ namespace Beagle.Daemon {
 			}
 		}
 
-		override public string GetSnippet (string[] query_terms, Hit hit) 
+		override public ISnippetReader GetSnippet (string[] query_terms, Hit hit, bool full_text, int ctx_length, int snp_length) 
 		{
 			if (text_cache == null)
 				return null;
@@ -66,10 +63,7 @@ namespace Beagle.Daemon {
 			if (reader == null)
 				return null;
 			
-			string snippet = SnippetFu.GetSnippet (query_terms, reader);
-			reader.Close ();
-			
-			return snippet;
+			return SnippetFu.GetSnippet (query_terms, reader, full_text, ctx_length, snp_length);
 		}
 
 		override protected bool HitIsValid (Uri uri)
